//
// JPEG Cam, Motion Detection, speichern auf SD Card
// Sketch basiert auf dem Adafuit Beispiel MotionDetect (angepasst für Arduino Uno)
// File: tb_sensor_jpeg_cam_motion_detection.ino
// Datum: 9.10.16
// Board: Arduino Uno
//
// 
// This is a motion-detect camera sketch using the Adafruit VC0706 library.
// On start, the Arduino will find the camera and SD card and turn
// on motion detection.  If motion is detected, the camera will
// snap a photo, saving it to the SD card.
// Public domain.

#include <Adafruit_VC0706.h>
#include <SPI.h>
#include <SD.h>

// Softwaremässige serielle Schnittstelle
#include <SoftwareSerial.h>         

// Chip Select (Arduino Uno)
#define chipSelect 10

// Kamera an Arduino Uno: Kamera TX an D2, Kamera RX an D3
// SoftwareSerial(RX, TX)
SoftwareSerial cameraconnection = SoftwareSerial(2, 3);

// Kamera-Objekt erstellen
Adafruit_VC0706 cam = Adafruit_VC0706(&cameraconnection);

void setup() 
{
  // SPI SS als Ausgang
  if(chipSelect != 10)
  {
    pinMode(10, OUTPUT);
  }
  // Serielle Schnittstelle
  Serial.begin(9600);
  Serial.println("VC0706 Kamera Test");
  
  // prüfen, ob Karte verfügbar
  if (!SD.begin(chipSelect)) 
  {
    Serial.println("Zugriff auf SD-Karte fehlerhaft oder Karte nicht vorhanden");
    // sonst nichts tun
    return;
  }  
  
  // Kamera abfragen
  if (cam.begin()) {
    Serial.println("Kamera gefunden...");
  } else {
    Serial.println("ERROR: keine Kamera gefunden");
    return;
  }
  // Version von Kamera ausgeben
  char *reply = cam.getVersion();
  if (reply == 0) {
    Serial.print("ERROR: Version kann nicht ermittelt werden");
  } else {
    Serial.println("-----------------");
    Serial.print(reply);
    Serial.println("-----------------");
  }

  // Bildgrösse definieren (Standard mittel)
  // gross
  //cam.setImageSize(VC0706_640x480);  
  // mittel     
  cam.setImageSize(VC0706_320x240);
  // klein         
  //cam.setImageSize(VC0706_160x120);       

  // Bildgrösse lesen und ausgeben
  uint8_t imgsize = cam.getImageSize();
  Serial.print("Bildgrösse: ");
  if (imgsize == VC0706_640x480) Serial.println("640x480");
  if (imgsize == VC0706_320x240) Serial.println("320x240");
  if (imgsize == VC0706_160x120) Serial.println("160x120");


  // Bewegungserkennung ein- oder ausschalten
  // (entsprechende Zeile auskommentieren)
  // Ein
  cam.setMotionDetect(true);
  // Aus
  //cam.setMotionDetect(false);

  // prüfen, ob Bewegungserkennung Ein
  Serial.print("Bewegungserkennung...");
  if (cam.getMotionDetect()) 
    Serial.println("Ein");
  else 
    Serial.println("Aus");
}


void loop() 
{
  // falls Bewegung erkannt
  if (cam.motionDetected())
  {
   Serial.println("Bewegung...");   
   cam.setMotionDetect(false);
   
  if (! cam.takePicture()) 
    Serial.println("ERROR: Bild konnte nicht erstellt werden...");
  else 
    Serial.println("Bild erstellt...");
  
  char filename[13];
  // Dateiname
  strcpy(filename, "IMAGE00.JPG");
  for (int i = 0; i < 100; i++) {
    filename[5] = '0' + i/10;
    filename[6] = '0' + i%10;
    // break, falls Karte nicht vorhanden
    if (! SD.exists(filename)) {
      break;
    }
  }
  
  File imgFile = SD.open(filename, FILE_WRITE);
  
  uint16_t jpglen = cam.frameLength();
  Serial.print(jpglen, DEC);
  Serial.println(" Bild (Byte)");
 
  Serial.print("Bild schreiben "); Serial.print(filename);
  
  while (jpglen > 0) {
    // immer 32 Bytes lesen
    uint8_t *buffer;
    uint8_t bytesToRead = min(32, jpglen);
    buffer = cam.readPicture(bytesToRead);
    imgFile.write(buffer, bytesToRead);
    jpglen -= bytesToRead;
  }
  imgFile.close();
  Serial.println("Erstellt...");
  cam.resumeVideo();
  cam.setMotionDetect(true);
 }
}

