//
// Abstandmessung mit HC-SR04 Ultraschall-Sensor und LED-Anzeige
// (basierend auf Beispiel Sparkfun)
//

/**
 * HC-SR04 Demo
 * Demonstration of the HC-SR04 Ultrasonic Sensor
 * Date: August 3, 2016
 * License:
 *  Public Domain
 */

// Sensor Pins
const int TRIG_PIN = 9;
const int ECHO_PIN = 10;

// LED-Pins
int PinLED1 = 7;
int PinLED10 = 6;
int PinLED20 = 5;
int PinLED30 = 4;
int PinLED40 = 3;
int PinLED50 = 2;


// Distanzen über 400cm (23200 us pulse) sind "out of range"
const unsigned int MAX_DIST = 23200;

void setup() 
{
  // Trigger für Start Messung
  pinMode(TRIG_PIN, OUTPUT);
  digitalWrite(TRIG_PIN, LOW);

  // LED für Abstandsanzeige
  pinMode(PinLED1, OUTPUT);
  pinMode(PinLED10, OUTPUT);
  pinMode(PinLED20, OUTPUT);
  pinMode(PinLED30, OUTPUT);
  pinMode(PinLED40, OUTPUT);
  pinMode(PinLED50, OUTPUT);

  // Serielle Schnittstelle
  Serial.begin(9600);
}

void loop() 
{
  unsigned long t1;
  unsigned long t2;
  unsigned long pulse_width;
  float cm;
  float inches;


  // Trigger auf HIGH für min. 10us
  digitalWrite(TRIG_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);

  // Warten auf Signal an Echo-Pin
  while ( digitalRead(ECHO_PIN) == 0 );

  // Note: the micros() counter will overflow after ~70 min
  // Messung von Pulslänge an Echo-Pin
  t1 = micros();
  while ( digitalRead(ECHO_PIN) == 1);
  t2 = micros();
  pulse_width = t2 - t1;

  // Berechnung von Distanz in cm und inch
  cm = pulse_width / 58.0;
  inches = pulse_width / 148.0;

  // Ausgabe auf serielle Schnittstelle
  if ( pulse_width > MAX_DIST ) 
  {
    Serial.println("ERROR: Out of range");
  } 
  else 
  {
    Serial.print("Abstand: ");
    Serial.print(cm);
    Serial.print(" cm, ");
    Serial.print(inches);
    Serial.println(" in");

    // Ansteuerung der Abstands-LED
    if (cm >= 1)
    {
      PORTD = B10000000;
      }
    if (cm >= 10)
    {
      PORTD = B11000000;
      }
    if (cm >= 20)
    {
      PORTD = B11100000;
      }
    if (cm >= 30)
    {
      PORTD = B11110000;
      }
    if (cm >= 40)
    {
      PORTD = B11111000;
      }
    if (cm >= 50)
    {
      PORTD = B11111100;
      }
  }
    // warten bis nächste Messung
  delay(60);
}
